@@ -33,7 +33,7 @@ import json
 import logging
 from os import path
 
-from packaging import version as packaging_version
+from packvers import version as packaging_version
 import requests
 from requests.exceptions import ConnectionError
 
@@ -67,25 +67,6 @@ def total_seconds(td):
         return val / 10 ** 6
 
 
-def is_outdated(release_date):
-    """
-    Return True if 90 days have passed since `release_date` datetime object.
-
-    For example:
-
-    >>> release_date = datetime.datetime(2020, 9, 23)
-    >>> is_outdated(release_date)
-    True
-    >>> release_date =  datetime.datetime.utcnow()
-    >>> is_outdated(release_date)
-    False
-    """
-    current_time = datetime.datetime.utcnow()
-    seconds_since_last_check = total_seconds(current_time - release_date)
-    ninety_days = 90 * 24 * 60 * 60
-    return seconds_since_last_check > ninety_days
-
-
 class VersionCheckState:
 
     def __init__(self):
@@ -135,22 +116,7 @@ def build_outdated_message(installed_version, release_date, newer_version=''):
     return msg
 
 
-def check_scancode_version_locally(
-    installed_version=scancode_version,
-    release_date=scancode_release_date,
-):
-    """
-    Return a message to display if outdated or None. Work offline, without a
-    PyPI remote check.
-    """
-    if is_outdated(release_date):
-        return build_outdated_message(
-            installed_version=installed_version,
-            release_date=release_date,
-        )
-
-
-def check_scancode_version_remotely(
+def check_scancode_version(
     installed_version=scancode_version,
     release_date=scancode_release_date,
     new_version_url='https://pypi.org/pypi/scancode-toolkit/json',
@@ -188,8 +154,8 @@ def fetch_newer_version(
     State is stored in the scancode_cache_dir.
     If `force` is True, redo a PyPI remote check.
     """
-    installed_version = packaging_version.parse(installed_version)
     try:
+        installed_version = packaging_version.parse(installed_version)
         state = VersionCheckState()
 
         current_time = datetime.datetime.utcnow()
@@ -219,16 +185,6 @@ def fetch_newer_version(
 
         latest_version = packaging_version.parse(latest_version)
 
-        # Our git version string is not PEP 440 compliant, and thus improperly
-        # parsed via most 3rd party version parsers. We handle this case by
-        # pulling out the "base" release version by split()-ting on "post".
-        #
-        # For example, "3.1.2.post351.850399ba3" becomes "3.1.2"
-        if isinstance(installed_version, packaging_version.LegacyVersion):
-            installed_version = installed_version.split('post')
-            installed_version = installed_version[0]
-            installed_version = packaging_version.parse(installed_version)
-
         # Determine if our latest_version is older
         if (installed_version < latest_version
         and installed_version.base_version != latest_version.base_version):
